@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2013, University of Amsterdam
+    Copyright (c)  1985-2019, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -80,8 +81,7 @@ static void	collectSiblingsTime(void);
 
 int
 activateProfiler(prof_status active ARG_LD)
-{
-  int i;
+{ int i;
   PL_local_data_t *profiling;
 
   PL_LOCK(L_THREAD);
@@ -512,6 +512,7 @@ typedef struct prof_ref
   uintptr_t sibling_ticks;
   uintptr_t calls;			/* calls to/from this predicate */
   uintptr_t redos;			/* redos to/from this predicate */
+  uintptr_t exits;			/* exits to/from this predicate */
 } prof_ref;
 
 
@@ -521,6 +522,7 @@ typedef struct
   uintptr_t sibling_ticks;
   uintptr_t calls;
   uintptr_t redos;
+  uintptr_t exits;
   uintptr_t recur;
   prof_ref *callers;
   prof_ref *callees;
@@ -556,6 +558,7 @@ add_parent_ref(node_sum *sum,
   { if ( r->handle == handle && r->cycle == cycle )
     { r->calls += self->calls;
       r->redos += self->redos;
+      r->exits += self->exits;
       r->ticks += self->ticks;
       r->sibling_ticks += self->sibling_ticks;
 
@@ -566,6 +569,7 @@ add_parent_ref(node_sum *sum,
   r = allocHeapOrHalt(sizeof(*r));
   r->calls = self->calls;
   r->redos = self->redos;
+  r->exits = self->exits;
   r->ticks = self->ticks;
   r->sibling_ticks = self->sibling_ticks;
   r->handle = handle;
@@ -606,6 +610,7 @@ add_sibling_ref(node_sum *sum, call_node *sibling, int cycle)
   { if ( r->handle == sibling->handle && r->cycle == cycle )
     { r->calls += sibling->calls;
       r->redos += sibling->redos;
+      r->exits += sibling->exits;
       r->ticks += sibling->ticks;
       r->sibling_ticks += sibling->sibling_ticks;
 
@@ -616,6 +621,7 @@ add_sibling_ref(node_sum *sum, call_node *sibling, int cycle)
   r = allocHeapOrHalt(sizeof(*r));
   r->calls = sibling->calls;
   r->redos = sibling->redos;
+  r->exits = sibling->exits;
   r->ticks = sibling->ticks;
   r->sibling_ticks = sibling->sibling_ticks;
   r->handle = sibling->handle;
@@ -667,10 +673,10 @@ unify_relatives(term_t list, prof_ref *r ARG_LD)
 { term_t tail = PL_copy_term_ref(list);
   term_t head = PL_new_term_ref();
   term_t tmp = PL_new_term_ref();
-  static functor_t FUNCTOR_node6;
+  static functor_t FUNCTOR_node7;
 
-  if ( !FUNCTOR_node6 )
-    FUNCTOR_node6 = PL_new_functor(PL_new_atom("node"), 6);
+  if ( !FUNCTOR_node7 )
+    FUNCTOR_node7 = PL_new_functor(PL_new_atom("node"), 7);
 
   for( ; r; r=r->next)
   { int rc;
@@ -687,13 +693,14 @@ unify_relatives(term_t list, prof_ref *r ARG_LD)
       rc=(*r->type->unify)(tmp, r->handle);
 
     if ( !rc ||
-	 !PL_unify_term(head, PL_FUNCTOR, FUNCTOR_node6,
+	 !PL_unify_term(head, PL_FUNCTOR, FUNCTOR_node7,
 			PL_TERM, tmp,
 			PL_INT,  r->cycle,
 			PL_LONG, r->ticks,
 			PL_LONG, r->sibling_ticks,
 			PL_LONG, r->calls,
-			PL_LONG, r->redos) )
+			PL_LONG, r->redos,
+		        PL_LONG, r->exits) )
       fail;
   }
 
@@ -744,7 +751,7 @@ $prof_procedure_data(+Procedure,
 
 
 static
-PRED_IMPL("$prof_procedure_data", 7, prof_procedure_data, PL_FA_TRANSPARENT)
+PRED_IMPL("$prof_procedure_data", 8, prof_procedure_data, PL_FA_TRANSPARENT)
 { PRED_LD
   void *handle;
   node_sum sum;
@@ -767,8 +774,9 @@ PRED_IMPL("$prof_procedure_data", 7, prof_procedure_data, PL_FA_TRANSPARENT)
 	 PL_unify_integer(A3, sum.sibling_ticks) &&
 	 PL_unify_integer(A4, sum.calls) &&
 	 PL_unify_integer(A5, sum.redos) &&
-	 unify_relatives(A6, sum.callers PASS_LD) &&
-	 unify_relatives(A7, sum.callees PASS_LD)
+	 PL_unify_integer(A6, sum.exits) &&
+	 unify_relatives(A7, sum.callers PASS_LD) &&
+	 unify_relatives(A8, sum.callees PASS_LD)
        );
 
   free_relatives(sum.callers);
@@ -889,6 +897,19 @@ profCall(Definition handle)
     Make a call from the current node to handle.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+#ifdef O_DEBUG
+static char *
+node_name(call_node *n)
+{ static char buf[100];
+
+  if ( n->type == &prof_default_type )
+    return predicateName(n->handle);
+
+  Ssprintf(buf, "%p", n->handle);
+  return buf;
+}
+#endif
+
 static call_node *
 prof_call(void *handle, PL_prof_type_t *type ARG_LD)
 { call_node *node = LD->profile.current;
@@ -901,7 +922,7 @@ prof_call(void *handle, PL_prof_type_t *type ARG_LD)
       { node->calls++;
 	LD->profile.current = node;
 	DEBUG(MSG_PROF_CALLTREE,
-	      Sdprintf("existing root %p\n", LD->profile.current));
+	      Sdprintf("Call: existing root %s\n", node_name(node)));
 
 	LD->profile.accounting = FALSE;
 	return node;
@@ -920,7 +941,8 @@ prof_call(void *handle, PL_prof_type_t *type ARG_LD)
     LD->profile.roots = node;
     LD->profile.current = node;
     LD->profile.accounting = FALSE;
-    DEBUG(MSG_PROF_CALLTREE, Sdprintf("new root %p\n", node));
+    DEBUG(MSG_PROF_CALLTREE,
+	  Sdprintf("Call: new root %s\n", node_name(node)));
 
     return node;
   }
@@ -928,7 +950,8 @@ prof_call(void *handle, PL_prof_type_t *type ARG_LD)
 					/* straight recursion */
   if ( node->handle == handle )
   { node->recur++;
-    DEBUG(MSG_PROF_CALLTREE, Sdprintf("direct recursion\n"));
+    DEBUG(MSG_PROF_CALLTREE,
+	  Sdprintf("Call: direct recursion on %s\n", node_name(node)));
     LD->profile.accounting = FALSE;
     return node;
   } else				/* from same parent */
@@ -941,19 +964,20 @@ prof_call(void *handle, PL_prof_type_t *type ARG_LD)
       { node->recur++;
 
 	LD->profile.current = node;
-	DEBUG(MSG_PROF_CALLTREE, Sdprintf("indirect recursion\n"));
+	DEBUG(MSG_PROF_CALLTREE,
+	      Sdprintf("Call: indirect recursion on %s\n", node_name(node)));
 	LD->profile.accounting = FALSE;
 	return node;
       }
     }
   }
 
-
   for(node=LD->profile.current->siblings; node; node=node->next)
   { if ( node->handle == handle )
     { LD->profile.current = node;
       node->calls++;
-      DEBUG(MSG_PROF_CALLTREE, Sdprintf("existing child\n"));
+      DEBUG(MSG_PROF_CALLTREE,
+	    Sdprintf("Call: existing child %s\n", node_name(node)));
       LD->profile.accounting = FALSE;
       return node;
     }
@@ -970,7 +994,8 @@ prof_call(void *handle, PL_prof_type_t *type ARG_LD)
   node->next = LD->profile.current->siblings;
   LD->profile.current->siblings = node;
   LD->profile.current = node;
-  DEBUG(MSG_PROF_CALLTREE, Sdprintf("new child\n"));
+  DEBUG(MSG_PROF_CALLTREE,
+	Sdprintf("Call: new child %s\n", node_name(node)));
   LD->profile.accounting = FALSE;
 
   return node;
@@ -1005,7 +1030,9 @@ profResumeParent(struct call_node *node ARG_LD)
 
   LD->profile.accounting = TRUE;
   for(n=LD->profile.current; n && n != node; n=n->parent)
-  { n->exits++;
+  { DEBUG(MSG_PROF_CALLTREE,
+	  Sdprintf("Exit: %s\n", node_name(n)));
+    n->exits++;
   }
   LD->profile.accounting = FALSE;
 
@@ -1028,7 +1055,23 @@ profRedo(struct call_node *node ARG_LD)
     return;
 
   if ( node )
-  { node->redos++;
+  { if ( LD->profile.current )
+    { struct call_node *n;
+
+      DEBUG(MSG_PROF_CALLTREE,
+	    Sdprintf("Redo: on %s; current is %s\n",
+		     node_name(node), node_name(LD->profile.current) ));
+
+      for(n=node; n && n != LD->profile.current; n = n->parent)
+      { DEBUG(MSG_PROF_CALLTREE,
+	      Sdprintf("Redo: %s\n", node_name(n)));
+	n->redos++;
+      }
+    } else
+    { DEBUG(MSG_PROF_CALLTREE,
+	    Sdprintf("Redo: %s\n", node_name(node)));
+      node->redos++;
+    }
   }
   LD->profile.current = node;
 }
@@ -1187,7 +1230,7 @@ PRED_IMPL("$profile", 2, profile, PL_FA_TRANSPARENT)
 }
 
 static
-PRED_IMPL("$prof_procedure_data", 7, prof_procedure_data, PL_FA_TRANSPARENT)
+PRED_IMPL("$prof_procedure_data", 8, prof_procedure_data, PL_FA_TRANSPARENT)
 { return notImplemented("$prof_procedure_data", 7);
 }
 
@@ -1242,7 +1285,7 @@ BeginPredDefs(profile)
   PRED_DEF("reset_profiler", 0, reset_profiler, 0)
   PRED_DEF("$prof_node", 7, prof_node, 0)
   PRED_DEF("$prof_sibling_of", 2, prof_sibling_of, PL_FA_NONDETERMINISTIC)
-  PRED_DEF("$prof_procedure_data", 7, prof_procedure_data, PL_FA_TRANSPARENT)
+  PRED_DEF("$prof_procedure_data", 8, prof_procedure_data, PL_FA_TRANSPARENT)
   PRED_DEF("$prof_statistics", 5, prof_statistics, 0)
 #ifdef O_PROF_PENTIUM
   PRED_DEF("show_pentium_profile", 0, show_pentium_profile, 0)
