@@ -37,6 +37,7 @@
 #include "pl-incl.h"
 #include "pl-dbref.h"
 #include "pl-event.h"
+#include "pl-tabling.h"
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Source administration. The core object is  SourceFile, which keeps track
@@ -1088,7 +1089,7 @@ associateSource(SourceFile sf, Procedure proc)
 }
 
 
-#define P_ATEND	(P_VOLATILE|P_PUBLIC|P_ISO|P_NOPROFILE|P_NON_TERMINAL|P_TABLED)
+#define P_ATEND	(P_VOLATILE|P_PUBLIC|P_ISO|P_NOPROFILE|P_NON_TERMINAL)
 
 int
 setAttrProcedureSource(SourceFile sf, Procedure proc,
@@ -1269,6 +1270,21 @@ delete_old_predicate(SourceFile sf, Procedure proc)
 { Definition def = proc->definition;
   size_t deleted;
 
+  if ( def->functor->functor == FUNCTOR_dtabled2 )
+  { GET_LD
+    ClauseRef c;
+
+    acquire_def(def);
+    for(c = def->impl.clauses.first_clause; c; c = c->next)
+    { Clause cl = c->value.clause;
+
+      if ( false(cl, CL_ERASED) &&
+	   GLOBALLY_VISIBLE_CLAUSE(cl, global_generation()) )
+	untable_from_clause(cl);
+    }
+    release_def(def);
+  }
+
   deleted = removeClausesPredicate(
 		def,
 		true(def, P_MULTIFILE) ? sf->index : 0,
@@ -1328,6 +1344,9 @@ delete_pending_clauses(SourceFile sf, Definition def, p_reload *r ARG_LD)
     if ( true(r->predicate, P_MULTIFILE|P_DYNAMIC) && c->owner_no != sf->index )
       continue;
 
+    if ( def->functor->functor == FUNCTOR_dtabled2 )
+      untable_from_clause(c);
+
     c->generation.erased = rl->reload_gen;
     set(r, P_MODIFIED);
     DEBUG(MSG_RECONSULT_CLAUSE,
@@ -1339,6 +1358,34 @@ delete_pending_clauses(SourceFile sf, Definition def, p_reload *r ARG_LD)
 }
 
 
+static size_t
+end_reconsult_proc(SourceFile sf, Procedure proc, p_reload *r ARG_LD)
+{ size_t dropped_access = 0;
+
+  DEBUG(MSG_RECONSULT_CLAUSE,
+	Sdprintf("Fixup %s\n", predicateName(proc->definition)));
+
+  if ( false(r, P_NEW|P_NO_CLAUSES) )
+  { Definition def = proc->definition;
+
+    delete_pending_clauses(sf, def, r PASS_LD);
+    fix_attributes(sf, def, r PASS_LD);
+    reconsultFinalizePredicate(sf->reload, def, r PASS_LD);
+  } else
+  { dropped_access++;
+    if ( true(r, P_NO_CLAUSES) )
+    { Definition def = proc->definition;
+      fix_attributes(sf, def, r PASS_LD);
+    }
+  }
+  if ( r->args )
+    freeHeap(r->args, 0);
+  freeHeap(r, sizeof(*r));
+
+  return dropped_access;
+}
+
+
 static int
 endReconsult(SourceFile sf)
 { GET_LD
@@ -1347,28 +1394,14 @@ endReconsult(SourceFile sf)
   if ( (reload=sf->reload) )
   { size_t accessed_preds = reload->procedures->size;
 
+    delayEvents();
     delete_old_predicates(sf);
 
     for_table(reload->procedures, n, v,
 	      { Procedure proc = n;
 		p_reload *r = v;
 
-		if ( false(r, P_NEW|P_NO_CLAUSES) )
-		{ Definition def = proc->definition;
-
-		  delete_pending_clauses(sf, def, r PASS_LD);
-		  fix_attributes(sf, def, r PASS_LD);
-		  reconsultFinalizePredicate(reload, def, r PASS_LD);
-		} else
-		{ accessed_preds--;
-		  if ( true(r, P_NO_CLAUSES) )
-		  { Definition def = proc->definition;
-		    fix_attributes(sf, def, r PASS_LD);
-		  }
-		}
-		if ( r->args )
-		  freeHeap(r->args, 0);
-		freeHeap(r, sizeof(*r));
+		accessed_preds -= end_reconsult_proc(sf, proc, r PASS_LD);
 	      });
 
     popNPredicateAccess(accessed_preds);
@@ -1395,6 +1428,8 @@ endReconsult(SourceFile sf)
     LD->gen_reload = GEN_INVALID;
 
     pl_garbage_collect_clauses();
+    if ( sendDelayedEvents(TRUE) < 0 )
+      return FALSE;
   }
 
   return TRUE;
