@@ -1537,12 +1537,18 @@ typedef struct
   tmp_buffer   choicepoints;	/* Stack of trie state choicepoints */
 } trie_gen_state;
 
+typedef struct desc_tstate
+{ Word  term;
+  size_t size;
+} desc_tstate;
+
 typedef struct
 { Word	       term;		/* Term we are descending */
+  size_t       size;		/* Size of the current node */
   int	       compound;	/* Initialized for compound */
   int	       prune;		/* Use for pruning */
   segstack     stack;		/* Stack for argument handling */
-  Word	       buffer[64];	/* Quick buffer for stack */
+  desc_tstate  buffer[64];	/* Quick buffer for stack */
 } descent_state;
 
 static void
@@ -1592,19 +1598,27 @@ get_key(trie_gen_state *state, descent_state *dstate, word *key ARG_LD)
   { return FALSE;
   } else if ( isTerm(*p) )
   { Functor f = valueTerm(*p);
+    desc_tstate dts;
 
     *key = f->definition;
-    if ( !dstate->compound )
-    { dstate->compound = TRUE;
-      initSegStack(&dstate->stack, sizeof(Word),
-		   sizeof(dstate->buffer), dstate->buffer);
+    if ( dstate->size > 1 )
+    { if ( !dstate->compound )
+      { dstate->compound = TRUE;
+	initSegStack(&dstate->stack, sizeof(desc_tstate),
+		     sizeof(dstate->buffer), dstate->buffer);
+      }
+      dts.term = dstate->term+1;
+      dts.size = dstate->size-1;
+      pushSegStack(&dstate->stack, dts, desc_tstate);
+      DEBUG(MSG_TRIE_GEN,
+	    Sdprintf("Pushed %p, size %zd\n", dts.term, dts.size));
     }
-    pushSegStack(&dstate->stack, dstate->term+1, Word);
-    //Sdprintf("Pushed %p\n", dstate->term+1);
     dstate->term = &f->arguments[0];
+    dstate->size = arityFunctor(f->definition);
     return TRUE;
   } else
   { dstate->term++;
+    dstate->size--;
 
     if ( isIndirect(*p) )
     { *key = trie_intern_indirect(state->trie, *p, FALSE PASS_LD);
@@ -1624,6 +1638,12 @@ add_choice(trie_gen_state *state, descent_state *dstate, trie_node *node ARG_LD)
   int has_key;
   word k;
 
+  DEBUG(MSG_TRIE_GEN,
+	{ Word p;
+	  deRef2(dstate->term, p);
+	  Sdprintf("add_choice() for %s\n", print_val(*p, NULL));
+	});
+
   if ( dstate->prune )
   { if ( !(has_key = get_key(state, dstate, &k PASS_LD)) )
       dstate->prune = FALSE;
@@ -1648,12 +1668,18 @@ add_choice(trie_gen_state *state, descent_state *dstate, trie_node *node ARG_LD)
 	  ch->choice.any = NULL;
 
 	  if ( IS_TRIE_KEY_POP(children.key->key) && dstate->compound )
-	  { popSegStack(&dstate->stack, &dstate->term, Word);
-	    //Sdprintf("Popped %p\n", dstate->term);
+	  { desc_tstate dts;
+	    popSegStack(&dstate->stack, &dts, desc_tstate);
+	    dstate->term = dts.term;
+	    dstate->size = dts.size;
+	    DEBUG(MSG_TRIE_GEN,
+		  Sdprintf("Popped %p, left %zd\n", dstate->term, dstate->size));
 	  }
 	  break;
 	} else
+	{ DEBUG(MSG_TRIE_GEN, Sdprintf("Failed\n"));
 	  return NULL;
+	}
       case TN_HASHED:
 	if ( has_key && children.hash->var_keys == 0 )
 	{ trie_node *child;
@@ -1811,6 +1837,7 @@ trie_gen(term_t Trie, term_t Key, term_t Value,
 	  TRIE_STAT_INC(trie, gen_call);
 
 	  dstate.term     = valTermRef(Key);
+	  dstate.size     = 1;
 	  dstate.compound = FALSE;
 	  dstate.prune	  = TRUE;
 	  deRef(dstate.term);
@@ -1823,8 +1850,7 @@ trie_gen(term_t Trie, term_t Key, term_t Value,
 		 (ch->child->value || next_choice(state PASS_LD)) );
 	  clear_descent_state(&dstate);
 	  if ( !rc )
-	  { TRIE_STAT_INC(trie, gen_fail);
-	    clear_trie_state(state);
+	  { clear_trie_state(state);
 	    return FALSE;
 	  }
 	  break;
@@ -1875,8 +1901,7 @@ trie_gen(term_t Trie, term_t Key, term_t Value,
 
     if ( (!Value || unify_value(Value, n->value PASS_LD)) &&
 	 (!Data  || unify_data(Data, n, ctx PASS_LD)) )
-    { TRIE_STAT_INC(state->trie, gen_exit);
-      if ( next_choice(state PASS_LD) )
+    { if ( next_choice(state PASS_LD) )
       { if ( !state->allocated )
 	{ trie_gen_state *nstate = allocForeignState(sizeof(*state));
 	  TmpBuffer nchp = &nstate->choicepoints;
@@ -1903,8 +1928,7 @@ trie_gen(term_t Trie, term_t Key, term_t Value,
 	return TRUE;
       }
     } else
-    { TRIE_STAT_INC(state->trie, gen_fail);
-      Undo(fli_context->mark);
+    { Undo(fli_context->mark);
     }
 
 next:;
@@ -1947,19 +1971,20 @@ PRED_IMPL("$trie_property", 2, trie_property, 0)
 #ifdef O_TRIE_STATS
   static atom_t ATOM_lookup_count = 0;
   static atom_t ATOM_gen_call_count = 0;
-  static atom_t ATOM_gen_exit_count = 0;
-  static atom_t ATOM_gen_fail_count = 0;
+  static atom_t ATOM_invalidated = 0;
+  static atom_t ATOM_reevaluated = 0;
 
   if ( !ATOM_lookup_count )
   { ATOM_lookup_count   = PL_new_atom("lookup_count");
     ATOM_gen_call_count = PL_new_atom("gen_call_count");
-    ATOM_gen_exit_count = PL_new_atom("gen_exit_count");
-    ATOM_gen_fail_count = PL_new_atom("gen_fail_count");
+    ATOM_invalidated    = PL_new_atom("invalidated");
+    ATOM_reevaluated    = PL_new_atom("reevaluated");
   }
 #endif
 
   if ( get_trie(A1, &trie) )
   { atom_t name; size_t arity;
+    idg_node *idg;
 
     if ( PL_get_name_arity(A2, &name, &arity) && arity == 1 )
     { term_t arg = PL_new_term_ref();
@@ -2001,10 +2026,10 @@ PRED_IMPL("$trie_property", 2, trie_property, 0)
       { return PL_unify_int64(arg, trie->stats.lookups);
       } else if ( name == ATOM_gen_call_count)
       { return PL_unify_int64(arg, trie->stats.gen_call);
-      } else if ( name == ATOM_gen_exit_count)
-      { return PL_unify_int64(arg, trie->stats.gen_exit);
-      } else if ( name == ATOM_gen_fail_count)
-      { return PL_unify_int64(arg, trie->stats.gen_fail);
+      } else if ( name == ATOM_invalidated && (idg=trie->data.IDG))
+      { return PL_unify_int64(arg, idg->stats.invalidated);
+      } else if ( name == ATOM_reevaluated && (idg=trie->data.IDG))
+      { return PL_unify_int64(arg, idg->stats.reevaluated);
 #endif
       }
     }
@@ -2395,6 +2420,9 @@ create_trie_clause(Definition def, Clause *cp, trie_compile_state *state)
   memcpy(cl->codes, baseBuffer(&state->codes, code), sizeOfBuffer(&state->codes));
   *cp = cl;
 
+  ATOMIC_ADD(&GD->statistics.codes, cl->code_size);
+  ATOMIC_INC(&GD->statistics.clauses);
+
   return TRUE;
 }
 
